EXTENSIONS = [
    {
        'name': 'case02',
        'extension_class': '.extension.Case02Extension'
    }
]

VIEWS = [
    {
        'uri': '/',
        'view_class': '.view.Case02View'
    },
]

STATICS = [
    {
        'uri': '/static',
        'file_or_directory': './static'
    }
]
